package com.neusoft.smart.pos.dto.order.request;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.hibernate.validator.constraints.NotEmpty;

import javax.validation.constraints.DecimalMax;
import java.math.BigDecimal;

/**
 * @author tmg
 * @date 2020--05-13 10：43
 * @description 招行会员卡充值订单新增使用Request
 */
@Data
@ApiModel
public class NeusoftMemberRechargeRequest {
    @NotEmpty(message = "商户号不能为空")
    @ApiModelProperty(name = "mcode", value = "商户号", dataType = "String" ,example="15554512555")
    private String mcode;

    @NotEmpty(message = "签名不能为空")
    @ApiModelProperty(name = "sign", value = "签名",example="0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF", dataType = "String" ,required=true)
    private String sign;

    @ApiModelProperty(name = "memberCardNo", value = "会员卡号、东软员工号", dataType = "String" ,example="52525005")
    private String memberCardNo;

    //yyyy-mm-dd hh:mm:ss;例如：2020-04-17 16:00:00
    @ApiModelProperty(name = "time", value = "支付时间", dataType = "String" ,example="2020-04-17 16:00:00")
    private String time;

    @ApiModelProperty(name = "transNo", value = "招行APP支付唯一流水号", dataType = "String" ,example="52525005")
    private String transNo;

    @DecimalMax(value = "99999999.99",message = "订单金额非法")
    @ApiModelProperty(name = "amount", value = "订单总金额", dataType = "Long", example = "1898",required=true)
    private Long amount;

    private String nonce;

    private String timestamp;
}
