package com.neusoft.smart.pos.dto.payment.request;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

/**
 * @author zcf
 * @date 2018-01-22 10：43
 * @description 第三方支付单生成使用Request
 */
@Data
@ApiModel
public class PostGenerateOrderRequest {
	
	@ApiModelProperty(name = "mcode", value = "商户号(聚合支付平台提供)", dataType = "String" ,example="NEU226104452154865",required=true)
	private String mcode;

	@ApiModelProperty(name = "payType", value = "支付方式(@1:微信扫码支付@2:支付宝扫码支付@3:支付宝当面资金授权支付@4:银联二维码支付@5:微信商户主动扫码支付@6:支付宝商户主动扫码支付)", dataType = "String" ,example="1",required=true)
	private String payType;

	@ApiModelProperty(name = "orderAmount", value = "支付金额,单位分,不能包含小数", dataType = "Long" ,example="1000",required=true)
	private Long orderAmount;

    @ApiModelProperty(name = "leftAmount", value = "交易订单剩余未支付金额,单位分,不能包含小数", dataType = "Long" ,example="1000",required=true)
    private Long leftAmount;

	@ApiModelProperty(name = "orderNumber", value = "业务系统订单号(在业务系统中唯一)", dataType = "String" ,example="4200000073201",required=true)
	private String orderNumber;

	@ApiModelProperty(name = "timeExpire", value = "订单失效时间，格式为yyyyMMddHHmmss，有效时间至少5分钟,如果为空默认为2小时", dataType = "String" ,example="20180129172506",required=false)
	private String timeExpire;

	@ApiModelProperty(name = "description", value = "订单描述", dataType = "String" ,example="点餐支付订单",required=false)
	private String description;

	@ApiModelProperty(name = "notifyUrl", value = "支付成功回调url(如果为空则使用商户号默认配置)", dataType = "String" ,example="http://www.test.com/payNotify/neuPayNotify",required=false)
	private String notifyUrl;

	@ApiModelProperty(name = "scanCode", value = "扫码支付授权码", dataType = "String" ,example="120061098828009406",required=false)
	private String scanCode;

	@ApiModelProperty(name = "tradeType", value = "交易类型(只对微信支付有效MWEB)", dataType = "String" ,example="120061098828009406",required=false)
	private String tradeType;

	@ApiModelProperty(name = "ip", value = "IP地址", dataType = "String" ,example="10.209.96.189",required=false)
	private String ip;

	@ApiModelProperty(name = "openId", value = "用户openId", dataType = "String" ,example="213123123",required=false)
	private String openId;
	
	@ApiModelProperty(name = "deviceId", value = "设备ID", dataType = "String" ,example="213123123",required=false)
	private String deviceId;
	
	@ApiModelProperty(name = "storeId", value = "支付宝开放平台门店编号", dataType = "String")
	private String storeId;
	
	@ApiModelProperty(name = "pid", value = "分润商户PID", dataType = "String")
	private String pid;
	
	@ApiModelProperty(name = "sign", value = "签名(MD5加密字符串)", dataType = "String" ,example="5CFB108E5A9CAA78B141251BC2BD7F74",required=true)
	private String sign;

	@ApiModelProperty(name = "payInfoJsonString", value = "第三方支付所需信息", dataType = "String" ,example="{info:xxx}",required=false)
	private String payInfoJsonString;

}
