package com.neusoft.smart.pos.dto.trade.request;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.hibernate.validator.constraints.NotEmpty;

import javax.validation.constraints.DecimalMax;
import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.List;

/**
 * @author zcf
 * @date 2018-04-08 10：43
 * @description 会员卡充值订单新增使用Request
 */
@Data
@ApiModel
public class PostRechargeOrderRequest {

	@NotEmpty(message = "设备sn不能为空")
	@ApiModelProperty(name = "serialNo", value = "设备序列号", dataType = "String" ,example="15554512555")
	private String serialNo;

	@NotEmpty(message = "签名不能为空")
	@ApiModelProperty(name = "sign", value = "签名",example="0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF0123456789ABCDEF", dataType = "String" ,required=true)
	private String sign;

	@ApiModelProperty(name = "memberCardNo", value = "会员卡号、东软员工号", dataType = "String" ,example="52525005")
	private String memberCardNo;

	@ApiModelProperty(name = "memberName", value = "会员名称、东软员工姓名", dataType = "String" ,example="52525005")
	private String memberName;

	@ApiModelProperty(name = "cardNum", value = "会员卡编号(印刷,沈阳食堂使用)", dataType = "String" ,example="52525005")
	private String cardNum;

	@ApiModelProperty(name = "cardHidCode", value = "会员卡hid编码(卡内,沈阳食堂使用)", dataType = "String" ,example="52525005")
	private String cardHidCode;

    @DecimalMax(value = "99999999.99",message = "订单金额非法")
	@ApiModelProperty(name = "orderAmount", value = "订单总金额", dataType = "Long", example = "18.98",required=true)
	private BigDecimal orderAmount;

}
